################################################################
################################################################

library(rjags)

setwd("SET WORKING DIRECTORY HERE");

superiorData = read.csv("superiorAggBalance.csv", header = TRUE)
miHuronData = read.csv("miHuronAggBalance.csv", header = TRUE)

################################################################################################
################################################################################################
#### A. Set up data 
################################################################################################
################################################################################################

priorRange = which(miHuronData[,1] == 2004 & miHuronData[,2] == 12)   

priorSuperiorData = superiorData[1:priorRange,]
priorMiHuronData = miHuronData[1:priorRange,]
	
################################################################################################
################################################################################################
#### B. Parameters for prior distributions
################################################################################################
################################################################################################## 
#Let's look at priors and data from January months from 1950 to 2004

superiorPriorPrecipShape = miHuronPriorPrecipShape = NULL					
superiorPriorPrecipRate = miHuronPriorPrecipRate = NULL					
superiorEvapPriorMean = miHuronEvapPriorMean = NULL
superiorEvapPriorPrecision = miHuronEvapPriorPrecision = NULL
superiorRunoffLogPriorMean = miHuronRunoffLogPriorMean = NULL
superiorRunoffLogPriorPrecision = miHuronRunoffLogPriorPrecision = NULL
superiorOutflowPriorMean = miHuronOutflowPriorMean = NULL
superiorOutflowPriorPrecision = miHuronOutflowPriorPrecision = NULL
superiorDiversionPriorMean = miHuronDiversionPriorMean = NULL
superiorDiversionPriorPrecision = miHuronDiversionPriorPrecision = NULL

for(i in 1:12){

	## Over-lake precipitation; gamma distribution following Husak, et al (2007), IJOC
	# Note use of rate vs scale
	x.bar_s = mean(priorSuperiorData[priorSuperiorData[,2]==i,5])
	theta_s = log(x.bar_s)-mean(log(priorSuperiorData[priorSuperiorData[,2]==i,5]))
	superiorPriorPrecipShape[i] = 1/(4*theta_s)*(1+sqrt(1+4*theta_s/3))
	superiorPriorPrecipRate[i] = superiorPriorPrecipShape[i]/x.bar_s

	x.bar_m = mean(priorMiHuronData[priorMiHuronData[,2]==i,5])
	theta_m = log(x.bar_m)-mean(log(priorMiHuronData[priorMiHuronData[,2]==i,5]))
	miHuronPriorPrecipShape[i] = 1/(4*theta_m)*(1+sqrt(1+4*theta_m/3))
	miHuronPriorPrecipRate[i] = miHuronPriorPrecipShape[i]/x.bar_m

	## Over-lake evaporation
	superiorEvapPriorMean[i] = mean(priorSuperiorData[priorSuperiorData[,2]==i,7])
	superiorEvapPriorPrecision[i] =  0.5*(1/var(priorSuperiorData[priorSuperiorData[,2]==i,7]))					
	miHuronEvapPriorMean[i] = mean(priorMiHuronData[priorMiHuronData[,2]==i,7])
	miHuronEvapPriorPrecision[i] =  0.5*(1/var(priorMiHuronData[priorMiHuronData[,2]==i,7]))

	## Runoff
	superiorRunoffLogPriorMean[i] = mean(log(priorSuperiorData[priorSuperiorData[,2]==i,12]))
	superiorRunoffLogPriorPrecision[i] =  (1/var(log(priorSuperiorData[priorSuperiorData[,2]==i,12])))				
	miHuronRunoffLogPriorMean[i] = mean(log(priorMiHuronData[priorMiHuronData[,2]==i,12]))
	miHuronRunoffLogPriorPrecision[i] =  (1/var(log(priorMiHuronData[priorMiHuronData[,2]==i,12])))
	
	## Outflow
	superiorOutflowPriorMean[i] =  mean(priorSuperiorData[priorSuperiorData[,2]==i,10])
	superiorOutflowPriorPrecision[i] = 0.5*(1/var(priorSuperiorData[priorSuperiorData[,2]==i,10]))					
	miHuronOutflowPriorMean[i] =  mean(priorMiHuronData[priorMiHuronData[,2]==i,10])
	miHuronOutflowPriorPrecision[i] = 0.5*(1/var(priorMiHuronData[priorMiHuronData[,2]==i,10]))
	
	## Diversions
	superiorDiversionPriorMean[i] =  mean(priorSuperiorData[priorSuperiorData[,2]==i,9])
	superiorDiversionPriorPrecision[i] = (1/var(priorSuperiorData[priorSuperiorData[,2]==i,9]))					
	miHuronDiversionPriorMean[i] =  mean(priorMiHuronData[priorMiHuronData[,2]==i,15])
	miHuronDiversionPriorPrecision[i] = (1/var(priorMiHuronData[priorMiHuronData[,2]==i,15]))
}

# sd of 10 is precision of 0.01

################################################################################################
################################################################################################
#### C. Observations for likelihood function
################################################################################################
################################################################################################

posteriorRange = which(miHuronData[,1] == 2005 & miHuronData[,2] == 1):(dim(miHuronData)[1])

m = as.numeric(superiorData[posteriorRange,2])

rollPeriod = 12;

## Observations for Lake Superior ('s')
#y.delta_s	= as.numeric(superiorData[posteriorRange,4])
#y.cumd_s	= cumsum(y.delta_s)

ySuperiorBOM            = as.numeric(superiorData[posteriorRange,3])
ySuperiorDStore         = as.numeric(superiorData[posteriorRange,4])
ySuperiorRStore = rep(NA, length(ySuperiorDStore));
for(rp in rollPeriod:length(ySuperiorDStore)){
	ySuperiorRStore[rp] = sum(ySuperiorDStore[(rp-rollPeriod+1):rp]);
}
ySuperiorCStore         = cumsum(ySuperiorDStore);
ySuperiorPrecip1    	= as.numeric(superiorData[posteriorRange,5])
ySuperiorPrecip2    	= as.numeric(superiorData[posteriorRange,6])
ySuperiorEvap1    	    = as.numeric(superiorData[posteriorRange,7])
ySuperiorEvap2    	    = as.numeric(superiorData[posteriorRange,8])
ySuperiorRunoff1    	= as.numeric(superiorData[posteriorRange,12])
ySuperiorRunoff2    	= as.numeric(superiorData[posteriorRange,16])
ySuperiorOutflow1     	= as.numeric(superiorData[posteriorRange,10])
ySuperiorOutflow2     	= as.numeric(superiorData[posteriorRange,11])
ySuperiorDiversion1     = as.numeric(superiorData[posteriorRange,9])

yMiHuronBOM            = as.numeric(miHuronData[posteriorRange,3])
yMiHuronDStore         = as.numeric(miHuronData[posteriorRange,4])
yMiHuronRStore = rep(NA, length(yMiHuronDStore));
for(rp in rollPeriod:length(yMiHuronDStore)){
	yMiHuronRStore[rp] = sum(yMiHuronDStore[(rp-rollPeriod+1):rp]);
}
yMiHuronCStore         = cumsum(yMiHuronDStore);
yMiHuronPrecip1        = as.numeric(miHuronData[posteriorRange,5])
yMiHuronPrecip2        = as.numeric(miHuronData[posteriorRange,6])
yMiHuronEvap1          = as.numeric(miHuronData[posteriorRange,7])
yMiHuronEvap2          = as.numeric(miHuronData[posteriorRange,8])
yMiHuronRunoff1        = as.numeric(miHuronData[posteriorRange,12])
yMiHuronRunoff2        = as.numeric(miHuronData[posteriorRange,16])
yMiHuronOutflow1       = as.numeric(miHuronData[posteriorRange,10])
yMiHuronOutflow2       = as.numeric(miHuronData[posteriorRange,11])
yMiHuronDiversion1     = as.numeric(miHuronData[posteriorRange,15])

################################################################################################
################################################################################################
#### D. Call to WinBUGS
################################################################################################
################################################################################################
posteriorStartMonth = 1;
posteriorEndMonth = length(posteriorRange);

inputData <- list(  
	"posteriorStartMonth", "posteriorEndMonth",
	"m",
	"rollPeriod",
	"ySuperiorRStore", "yMiHuronRStore",	
	"superiorPriorPrecipShape", "superiorPriorPrecipRate", 
	"superiorEvapPriorMean", "superiorEvapPriorPrecision", 
	"superiorRunoffLogPriorMean", "superiorRunoffLogPriorPrecision", 
	"superiorOutflowPriorMean", "superiorOutflowPriorPrecision",
	"superiorDiversionPriorMean", "superiorDiversionPriorPrecision",
	"miHuronPriorPrecipShape", "miHuronPriorPrecipRate", 
	"miHuronEvapPriorMean", "miHuronEvapPriorPrecision", 
	"miHuronRunoffLogPriorMean", "miHuronRunoffLogPriorPrecision", 
	"miHuronOutflowPriorMean", "miHuronOutflowPriorPrecision",
	"miHuronDiversionPriorMean", "miHuronDiversionPriorPrecision",
	"ySuperiorPrecip1", "ySuperiorPrecip2", 
	"ySuperiorEvap1", "ySuperiorEvap2", 
	"ySuperiorRunoff1", "ySuperiorRunoff2", 
	"ySuperiorOutflow1", "ySuperiorOutflow2",
	"ySuperiorDiversion1",
	"yMiHuronPrecip1", "yMiHuronPrecip2", 
	"yMiHuronEvap1", "yMiHuronEvap2", 
	"yMiHuronRunoff1", "yMiHuronRunoff2", 
	"yMiHuronOutflow1", "yMiHuronOutflow2",
	"yMiHuronDiversion1"	
); 


inputDataCoreJAGS = list(
	"posteriorStartMonth"                   = posteriorStartMonth, 
	"posteriorEndMonth"                     = posteriorEndMonth,
	"rollPeriod"                            = rollPeriod,
	"m"                                     = m,
	"ySuperiorRStore"                       = ySuperiorRStore, 
	"yMiHuronRStore"	                    = yMiHuronRStore,
	"superiorPriorPrecipShape"              = superiorPriorPrecipShape,            
	"superiorPriorPrecipRate"               = superiorPriorPrecipRate,             
	"superiorEvapPriorMean"                 = superiorEvapPriorMean,               
	"superiorEvapPriorPrecision"            = superiorEvapPriorPrecision,          
	"superiorRunoffLogPriorMean"            = superiorRunoffLogPriorMean,          
	"superiorRunoffLogPriorPrecision"       = superiorRunoffLogPriorPrecision,     
	"superiorOutflowPriorMean"              = superiorOutflowPriorMean,            
	"superiorOutflowPriorPrecision"         = superiorOutflowPriorPrecision,       
	"superiorDiversionPriorMean"            = superiorDiversionPriorMean,          
	"superiorDiversionPriorPrecision"       = superiorDiversionPriorPrecision,     
	"miHuronPriorPrecipShape"               = miHuronPriorPrecipShape,             
	"miHuronPriorPrecipRate"                = miHuronPriorPrecipRate,              
	"miHuronEvapPriorMean"                  = miHuronEvapPriorMean,                
	"miHuronEvapPriorPrecision"             = miHuronEvapPriorPrecision,           
	"miHuronRunoffLogPriorMean"             = miHuronRunoffLogPriorMean,           
	"miHuronRunoffLogPriorPrecision"        = miHuronRunoffLogPriorPrecision,      
	"miHuronOutflowPriorMean"               = miHuronOutflowPriorMean,             
	"miHuronOutflowPriorPrecision"          = miHuronOutflowPriorPrecision,        
	"miHuronDiversionPriorMean"             = miHuronDiversionPriorMean,           
	"miHuronDiversionPriorPrecision"        = miHuronDiversionPriorPrecision,      
	"ySuperiorPrecip1"                      = ySuperiorPrecip1, 
	"ySuperiorPrecip2"                      = ySuperiorPrecip2 ,
	"ySuperiorEvap1"                        = ySuperiorEvap1,
	"ySuperiorEvap2"                        = ySuperiorEvap2 ,
	"ySuperiorRunoff1"                      = ySuperiorRunoff1, 
	"ySuperiorRunoff2"                      = ySuperiorRunoff2,
	"ySuperiorOutflow1"                     = ySuperiorOutflow1 ,
	"ySuperiorOutflow2"                     = ySuperiorOutflow2,
	"ySuperiorDiversion1"                   = ySuperiorDiversion1,
	"yMiHuronPrecip1"                       = yMiHuronPrecip1, 
	"yMiHuronPrecip2"                       = yMiHuronPrecip2 ,
	"yMiHuronEvap1"                         = yMiHuronEvap1, 
	"yMiHuronEvap2"                         = yMiHuronEvap2 ,
	"yMiHuronRunoff1"                       = yMiHuronRunoff1 ,
	"yMiHuronRunoff2"                       = yMiHuronRunoff2 ,
	"yMiHuronOutflow1"                      = yMiHuronOutflow1 ,
	"yMiHuronOutflow2"                      = yMiHuronOutflow2,
	"yMiHuronDiversion1"                    = yMiHuronDiversion1
);


paramsToMonitor <- c( 
	"superiorPrecip", "superiorEvap", "superiorRunoff", "superiorOutflow", "superiorDiversion",
	"miHuronPrecip", "miHuronEvap", "miHuronRunoff", "miHuronOutflow", "miHuronDiversion",
	"superiorRStore", "miHuronRStore",
	"superiorDStore", "miHuronDStore",
	"superiorR1YStore", "miHuronR1YStore",
	"superiorR5YStore", "miHuronR5YStore",
	"ySuperiorDStorePP", "yMiHuronDStorePP",
	"ySuperiorR1YStorePP", "yMiHuronR1YStorePP",
	"ySuperiorR5YStorePP", "yMiHuronR5YStorePP",
	"ySuperiorPrecip1Bias", "ySuperiorPrecip2Bias", 
	"ySuperiorEvap1Bias", "ySuperiorEvap2Bias", 
	"ySuperiorRunoff1Bias", "ySuperiorRunoff2Bias", 
	"ySuperiorOutflow1Bias", "ySuperiorOutflow2Bias",
	"ySuperiorDiversion1Bias",
	"yMiHuronPrecip1Bias", "yMiHuronPrecip2Bias", 
	"yMiHuronEvap1Bias", "yMiHuronEvap2Bias", 
	"yMiHuronRunoff1Bias", "yMiHuronRunoff2Bias", 
	"yMiHuronOutflow1Bias", "yMiHuronOutflow2Bias",
	"yMiHuronDiversion1Bias",
	"ySuperiorRStorePrec",	 
	"ySuperiorPrecip1Prec",	"ySuperiorPrecip2Prec", 
	"ySuperiorEvap1Prec", "ySuperiorEvap2Prec",
	"ySuperiorRunoff1Prec", "ySuperiorRunoff2Prec", 
	"ySuperiorOutflow1Prec", "ySuperiorOutflow2Prec",
	"ySuperiorDiversion1Prec",
	"yMiHuronRStorePrec",	
	"yMiHuronPrecip1Prec",	"yMiHuronPrecip2Prec", 
	"yMiHuronEvap1Prec", "yMiHuronEvap2Prec",
	"yMiHuronRunoff1Prec", "yMiHuronRunoff2Prec", 
	"yMiHuronOutflow1Prec", "yMiHuronOutflow2Prec",
	"yMiHuronDiversion1Prec",
	"ySuperiorPrecip1PP", 
	"ySuperiorPrecip2PP", 
	"ySuperiorEvap1PP", 
	"ySuperiorEvap2PP", 
	"ySuperiorRunoff1PP",
	"ySuperiorRunoff2PP",
	"ySuperiorOutflow1PP",
	"ySuperiorOutflow2PP",
	"ySuperiorDiversion1PP",
	"yMiHuronPrecip1PP",
	"yMiHuronPrecip2PP",
	"yMiHuronEvap1PP", 
	"yMiHuronEvap2PP",
	"yMiHuronRunoff1PP",
	"yMiHuronRunoff2PP",
	"yMiHuronOutflow1PP",
	"yMiHuronOutflow2PP",
	"yMiHuronDiversion1PP"
);

################################################################################################
#### -- Model of cumulative monthly observations
################################################################################################

halfIters = 1000;

date();
startTime = proc.time();

cat('ADAPTING SAMPLER TO MODEL...')

jMod = jags.model(
	file = "balanceClosureModel_SM_XXNF.bug.r",
	data = inputDataCoreJAGS,
	n.chains = 3
);
isAdapted = adapt(jMod);
while(!isAdapted){
	cat('More adapting...\n')
	adapt(jMod, 100)
}

# UPDATE FOR BURNIN
update(jMod, halfIters)

# SAMPLE
jSample = coda.samples(
	jMod, 
	paramsToMonitor, 
	halfIters, 
	ceiling(halfIters/1000), 
	na.rm=TRUE
)

cat('COMPUTING STATS...\n')
jSumStats = summary(jSample)
jSumStats_MSD = jSumStats$statistics[,1:2]
jSumStats_Q = jSumStats$quantiles
jSumEff = effectiveSize(jSample)

# GET R-HATS
cat('GETTING GELMAN-RUBIN STAT (COMPUTATIONALLY EXPENSIVE)...\n')
jRHat = gelman.diag(jSample, multivariate=FALSE)
jRHatEsts = jRHat$psrf

### Compose jSum
# Ensure row order

rn = rownames(jSumStats_MSD)
ro_MSD = match(rn, rownames(jSumStats_MSD))
ro_Q = match(rn, rownames(jSumStats_Q))
ro_eff = match(rn, names(jSumEff))
ro_rhat = match(rn, rownames(jRHatEsts))

jSum = cbind(
	jSumStats_MSD,
	jSumStats_Q[ro_Q,],
	jRHatEsts[ro_rhat,],
	jSumEff[ro_eff]
);

colnames(jSum)[10] = 'n.eff'

date();
endTime = proc.time() - startTime;

save.image("GL_L2SWBM_12NF_SM.RData");

